@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2008 Sun Microsystems Inc.
+# Copyright 2004-2009 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -20,10 +20,10 @@
 """Manages the default speech server for orca.  A script can use this
 as its speech server, or it can feel free to create one of its own."""
 
-__id__        = "$Id: speech.py 4469 2009-01-28 13:42:35Z wwalker $"
-__version__   = "$Revision: 4469 $"
-__date__      = "$Date: 2009-01-28 08:42:35 -0500 (Wed, 28 Jan 2009) $"
-__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
+__copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
@@ -38,6 +38,8 @@ import keynames
 import orca
 import orca_state
 import settings
+import sound
+import speech_generator
 
 from acss import ACSS
 from orca_i18n import _           # for gettext support
@@ -69,31 +71,19 @@ def getSpeechServerFactories():
                                  [''])
             factories.append(module)
         except:
-            debug.printException(debug.LEVEL_OFF)
+            debug.printException(debug.LEVEL_CONFIGURATION)
 
     return factories
 
-def init():
+def _initSpeechServer(moduleName, speechServerInfo):
 
     global _speechserver
 
-    if _speechserver:
+    if not moduleName:
         return
 
-    # First, find the factory module to use.  We will
-    # allow the user to give their own factory module,
-    # thus we look first in the global name space, and
-    # then we look in the orca namespace.
-    #
-    moduleName = settings.speechServerFactory
-
-    if moduleName:
-        debug.println(debug.LEVEL_CONFIGURATION,
-                      "Using speech server factory: %s" % moduleName)
-    else:
-        debug.println(debug.LEVEL_CONFIGURATION,
-                      "Speech not available.")
-        return
+    debug.println(debug.LEVEL_CONFIGURATION,
+                  "Trying to use speech server factory: %s" % moduleName)
 
     factory = None
     try:
@@ -119,6 +109,32 @@ def init():
     else:
         _speechserver = factory.SpeechServer.getSpeechServer()
 
+def init():
+
+    if _speechserver:
+        return
+
+    try:
+        moduleName = settings.speechServerFactory
+        _initSpeechServer(moduleName,
+                          settings.speechServerInfo)
+    except:
+        moduleNames = settings.speechFactoryModules
+        for moduleName in moduleNames:
+            if moduleName != settings.speechServerFactory:
+                try:
+                    _initSpeechServer(moduleName, None)
+                    if _speechserver:
+                        break
+                except:
+                    debug.printException(debug.LEVEL_SEVERE)
+
+    if _speechserver:
+        debug.println(debug.LEVEL_CONFIGURATION,
+                      "Using speech server factory: %s" % moduleName)
+    else:
+        debug.println(debug.LEVEL_CONFIGURATION, "Speech not available.")
+
 def __resolveACSS(acss=None):
     if acss:
         return acss
@@ -137,20 +153,35 @@ def sayAll(utteranceIterator, progressCallback):
             debug.println(debug.LEVEL_INFO, logLine)
             log.info(logLine)
 
-def speak(text, acss=None, interrupt=True):
-    """Speaks all queued text immediately.  If text is not None,
-    it is added to the queue before speaking.
+def _speak(text, acss, interrupt):
+    """Speaks the individual string using the given ACSS."""
 
-    Arguments:
-    - text:      optional text to add to the queue before speaking
-    - acss:      acss.ACSS instance; if None,
-                 the default voice settings will be used.
-                 Otherwise, the acss settings will be
-                 used to augment/override the default
-                 voice settings.
-    - interrupt: if True, stops any speech in progress before
-                 speaking the text
-    """
+    if settings.speakMultiCaseStringsAsWords:
+        text = _processMultiCaseString(text)
+    if orca_state.activeScript and orca_state.usePronunciationDictionary:
+        text = orca_state.activeScript.adjustForPronunciation(text)
+    if settings.speakMultiCaseStringsAsWords:
+        text = _processMultiCaseString(text)
+
+    logLine = "SPEECH OUTPUT: '" + text + "'"
+    extraDebug = ""
+    if acss in settings.voices.values():
+        for key in settings.voices:
+            if acss == settings.voices[key]:
+                if key != settings.DEFAULT_VOICE:
+                    extraDebug = " voice=%s" % key
+                break
+    debug.println(debug.LEVEL_INFO, logLine + extraDebug)
+    log.info(logLine + extraDebug)
+
+    if _speechserver:
+        _speechserver.speak(text, __resolveACSS(acss), interrupt)
+
+
+def speak(content, acss=None, interrupt=True):
+    """Speaks the given content.  The content can be either a simple
+    string or an array of arrays of objects returned by a speech
+    generator."""
 
     # We will not interrupt a key echo in progress.
     #
@@ -161,19 +192,56 @@ def speak(text, acss=None, interrupt=True):
     if settings.silenceSpeech:
         return
 
-    if settings.speakMultiCaseStringsAsWords:
-        text = _processMultiCaseString(text)
-    if orca_state.activeScript and orca_state.usePronunciationDictionary:
-        text = orca_state.activeScript.adjustForPronunciation(text)
-    if settings.speakMultiCaseStringsAsWords:
-        text = _processMultiCaseString(text)
+    subString = None
+    if isinstance(content, basestring):
+        subString = content
+    elif isinstance(content, list):
+        for element in content:
+            if isinstance(element, basestring):
+                if subString:
+                    subString += " " + element
+                else:
+                    subString = element
+            elif isinstance(element, speech_generator.Pause):
+                if subString:
+                    if subString[-1] != ".":
+                        subString += "."
+                    if settings.enablePauseBreaks:
+                        _speak(subString, acss, interrupt)
+                        subString = None
+            elif isinstance(element, speech_generator.LineBreak):
+                if subString:
+                    _speak(subString, acss, interrupt)
+                    subString = None
+            elif isinstance(element, sound.Sound):
+                if subString:
+                    _speak(subString, acss, interrupt)
+                    subString = None
+                element.play()
+            else:
+                if subString:
+                    _speak(subString, acss, interrupt)
+                subString = None
+                if isinstance(element, list):
+                    speak(element, acss, interrupt)
+                elif isinstance(element, ACSS):
+                    acss = ACSS(acss)
+                    acss.update(element)
+                else:
+                    debug.println(debug.LEVEL_WARNING,
+                                  "UNKNOWN speech element: '%s'" % element)
+    elif isinstance(content, sound.Sound):
+        content.play()
+    elif isinstance(content, (speech_generator.Pause,
+                              speech_generator.LineBreak)):
+        pass
+    else:
+        debug.printStack(debug.LEVEL_WARNING)
+        debug.println(debug.LEVEL_WARNING, 
+                      "bad content sent to speech.speak: '%s'" % repr(content))
 
-    logLine = "SPEECH OUTPUT: '" + text + "'"
-    debug.println(debug.LEVEL_INFO, logLine)
-    log.info(logLine)
-
-    if _speechserver:
-        _speechserver.speak(text, __resolveACSS(acss), interrupt)
+    if subString:
+        _speak(subString, acss, interrupt)
 
 def speakKeyEvent(event_string, eventType):
     """Speaks a key event immediately.
@@ -238,47 +306,6 @@ def isSpeaking():
     else:
         return False
 
-def speakUtterances(utterances, acss=None, interrupt=True):
-    """Speaks the given list of utterances immediately.
-
-    Arguments:
-    - list:      list of strings to be spoken
-    - acss:      acss.ACSS instance; if None,
-                 the default voice settings will be used.
-                 Otherwise, the acss settings will be
-                 used to augment/override the default
-                 voice settings.
-    - interrupt: if True, stop any speech currently in progress.
-    """
-
-    # We will not interrupt a key echo in progress.
-    #
-    if orca_state.lastKeyEchoTime:
-        interrupt = interrupt \
-            and ((time.time() - orca_state.lastKeyEchoTime) > 0.5)
-
-    if settings.silenceSpeech:
-        return
-    i = 0
-    length = len(utterances)
-    while ( i < length ):
-        if settings.speakMultiCaseStringsAsWords:
-            utterances[i] = _processMultiCaseString(utterances[i])
-        if orca_state.activeScript and orca_state.usePronunciationDictionary:
-            utterances[i] = orca_state.activeScript.adjustForPronunciation(\
-                            utterances[i])
-        if settings.speakMultiCaseStringsAsWords:
-            utterances[i] = _processMultiCaseString(utterances[i])
-        logLine = "SPEECH OUTPUT: '" + utterances[i] + "'"
-        debug.println(debug.LEVEL_INFO, logLine)
-        log.info(logLine)
-        i = i + 1
-
-    if _speechserver:
-        _speechserver.speakUtterances(utterances,
-                                       __resolveACSS(acss),
-                                       interrupt)
-
 def getInfo():
     info = None
     if _speechserver:
